#ifndef SLICEEDITOR_HH 
#define SLICEEDITOR_HH 1

#include "RequestManager.hh"
#include "Edit.hh"
#include <map>

using libSlice::SliceData;

typedef multimap<long, Edit *>   IndexEditMap;
typedef multimap<string, Edit *> SeqEditMap;

typedef map<string, long> SeqPosMap;
typedef map<long, long> IgnorePosMap;

typedef pair<long, Edit *> IndexEditPair;
typedef pair<string, Edit *> SeqEditPair;

class SliceEditor : public RequestManager
{
public:
  SliceEditor();
  ~SliceEditor();

  void addSlice(SliceData * slice);
  void addSliceRange(long & index, long & gindex);
  void addRequest(ConsensusRequest * request);
  void addReadData(ReadData * readData);

  int  printResults();

  void setVerifyIndex(int verifyIndex);
  void setVerifyEmpty(int verifyEmpty);
  void setCreateExtendedTcov(int createExtendedTcov);

  void processEditFile(const string & filename);

  static void handleOptions(Foundation * tf);

private:
  enum tcovType
  {
    origtype,
    edittype,
    emptytype
  };

  void writeExtendedTcov(SliceData * slice, tcovType type);
  void writeExtendedTcovReadTable();

  void loadConfigFile(ConfigSection * depth);
  bool doEditPosition(long gindex);

  bool m_ignoreLoaded;

  long m_newIndex;
  long m_newGindex;

  IgnorePosMap m_ignoreMap;

  IndexEditMap m_indexEdits;
  SeqEditMap   m_seqEdits;
  SeqPosMap    m_seqPos;

  static int m_verifyIndex;
  static int m_verifyEmpty;
  static int m_editFilterLevel;
  static int m_createExtendedTcov;
  static string m_outfilePrefix;
  static ofstream * m_origTcov;
  static ofstream * m_editTcov;
};

#endif
