#ifndef CONTIGTILE_HH
#define CONTIGTILE_HH 1

#include "Foundation.hh"
#include "ContigSequence.hh"

#include <vector>
#include <map>
#include <deque>
#include <set>

typedef vector<ContigSequence *> ReadVector;


struct ContigSeqCmp
{
  bool operator()(const ContigSequence * a,
                  const ContigSequence * b) const
  {
    return a->getReadCount() < b->getReadCount();
  }
};


typedef list<ContigSequence *> ContigSequenceSlice;
typedef deque<ContigSequenceSlice> ContigSequenceTile;


class ContigTile
{
public:
  ContigTile(ReadVector & readVector, long & gindex);
  ~ContigTile();

  void setTilingIterators(long gindex,
                          ContigSequenceSlice::iterator & start,
                          ContigSequenceSlice::iterator & end);

  void addReadToTiling(ContigSequence * read, 
                       long gindex, 
                       int numPositions=1);

  void removeReadFromTiling(ContigSequence * read, 
                            long gindex, 
                            int numPositions=1);

  ContigSequenceSlice & getTilingSlice(long gindex);

  void tileContig();

  void setDirty(long start, long length);

  void shiftTilingRight(int contigShiftDistance);
  void eraseTilingSlice(long gindex);
  void resizeTiling(int delta);
  void trimTilingLeft(int trimBases);
  void resetTiling();
  void insertTiling(long gindex, ContigSequenceSlice & tslice);
  void insertGapSlice(long gindex);

private:
  ReadVector & m_readVector;
  long & m_gindex;

  ContigSequenceTile m_tiling;

  long m_tileStart;
  long m_tileEnd;

  bool m_dirty;
};

#endif
