#include "ConsensusSequence.hh"
#include <stdio.h> // snprintf

using libSlice::ConsensusData;

ConsensusSequence::ConsensusSequence(int contigCount, 
                                     long index, 
                                     ostream & out,
                                     bool circular)
  : m_out(out)
{
  m_contigCount = contigCount;
  m_circular = circular;

  m_nuc = "";
  m_curOffset = 0;
  m_curIndex = 1;
}

ConsensusSequence::~ConsensusSequence()
{
  SequenceList::const_iterator i;
  for (i =  m_sequenceList.begin();
       i != m_sequenceList.end();
       i++)
  {
    delete *i;
  }
}

void ConsensusSequence::addSlice(SliceData * slice)
{
  char consensus = slice->getConsensus();
  ConsensusData result = slice->doCalculations();

  long coordinate = m_curIndex;
  if (consensus == '-') { coordinate--; }

  char buffer[16];
  snprintf(buffer, sizeof(buffer), "%02X", result.getQualityClass());

  addElement(consensus, coordinate, buffer);

  bool originSequence = m_sequenceMap.empty() && m_circular;

  unsigned short depth = slice->getDepth();
  for (unsigned short i = 0; i < depth; i++)
  {
    SequenceData * sd = getSequence(slice->getRead(i), 
                                    m_curOffset, 
                                    coordinate,
                                    originSequence);

    sd->addElement(m_curOffset, coordinate, slice->getBase(i));
  }

  m_curOffset++;
  if (consensus != '-') { m_curIndex++; }
}

void ConsensusSequence::addElement(char base, long index, char * qualclass)
{
  if (base != '-') { m_nucsequence += base; }

  m_nuc += base;
  m_qualclass += qualclass;
}


SequenceData * ConsensusSequence::getSequence(const ReadData * rd, 
                                              long offset,
                                              long index,
                                              bool originSequence)
{
  SequenceData * retval;
  SequenceMap::const_iterator i;

  i = m_sequenceMap.find(rd->getId());

  if (i == m_sequenceMap.end())
  {
    retval = new SequenceData(rd, offset, index, originSequence);
    m_sequenceMap[rd->getId()] = retval;
    m_sequenceList.push_back(retval);
  }
  else
  {
    retval = i->second;
  }

  return retval;
}



void ConsensusSequence::printContig(const string & asmblId)
{ 
  m_out << "##" << asmblId 
        << " "  << m_sequenceList.size() 
        << " "  << m_nuc.length() 
        << " bases, 00000000 checksum." << endl;

  SequenceData::print60(m_out, m_nuc);

  SequenceList::const_iterator i;
  for (i =  m_sequenceList.begin();
       i != m_sequenceList.end();
       i++)
  {
    (*i)->printContigRecord(m_out);
  }
}
