#include "FastaWriter.hh"

FastaWriter::FastaWriter() : RequestManager("FastaWriter")
{
  m_contigCount = 0;
  
  m_consensusSequence = "";
  m_length = 0;
}

string FastaWriter::m_asmblId("");


FastaWriter::~FastaWriter()
{

}

void FastaWriter::addSliceRange(long & index, long & gindex)
{
  if (m_contigCount)
  {
    printFasta();

    char buffer[32];
    snprintf(buffer, sizeof(buffer), "%d", m_contigCount);
    m_asmblId = buffer;
  }

  m_consensusSequence = "";
  m_length = 0;
  
  m_contigCount++;

  index = gindex = 0L;
}

void FastaWriter::addSlice(SliceData * slice)
{
  RequestManager::addSlice(slice);

  char consensus = slice->getConsensus();
  if (consensus != '-') { m_consensusSequence += consensus; }
  m_length++;

  delete slice;
}


int FastaWriter::printResults()
{
  if (m_contigCount)
  {
    printFasta();
  }

  return m_errorCount;
}

void FastaWriter::printFasta()
{
  m_out << ">" << m_asmblId 
        << " " << m_readManager->size()
        << " " << m_length
        << " bases, 00000000 checksum." << endl;

  print60(m_out, m_consensusSequence);
}

void FastaWriter::print60(ostream & out, const string & bases) const
{
  int count = 0;
  int printNewline = 1;

  string::const_iterator i;

  for (i = bases.begin();
       i != bases.end();
       i++)
  {
    out << *i;
    count++;
    printNewline = 1;

    if (count == 60)
    {
      out << endl;
      count = 0;

      printNewline = 0;
    }
  }

  if (printNewline) { out << endl; }
}

void FastaWriter::handleOptions(Foundation * tf)
{
  tf->getOptions()->addOptionResult("asmbl_id|i=s", &m_asmblId,
                                    "Specify the assembly id");
  RequestManager::handleOptions(tf);

  if (m_asmblId.empty()) { m_asmblId = "0"; }
}
