#include "ReadMerger.hh"
#include <cstdio>

ReadMerger::ReadMerger(Foundation * tf, 
                       ostream & out, 
                       ReadManager & readManager)
  : m_log(tf->getLogger()->getCategory("ReadMerger")),
    m_out(out),
    m_readManager(readManager)
{
  m_readCount = 0;
}

ReadMerger::~ReadMerger()
{

}

void ReadMerger::addReadData(ReadData * readData)
{
  const string origId = readData->getId();
  const string & seqname = readData->getSeqname();

  ReadMap::const_iterator i = m_readMap.find(seqname);
  if (i == m_readMap.end())
  {
    char buffer [16];
    snprintf(buffer, sizeof(buffer), "%d", m_readCount);
    m_readCount++;

    readData->resetId(buffer);
    readData->printXML(m_out);
    
    m_readMap[seqname] = (string) buffer;
    
    // Make a copy before inserting!
    m_readManager.addReadData(buffer, 
                              seqname.c_str(), 
                              readData->isReverseCompliment() ? "1" : "0",
                              readData->getChem().c_str());
  }
  else
  {
    m_log.pushMessageLevel(MESSAGE_LEVEL_DEBUG_LOW);
    m_log << "Duplicate seq(" << i->second << "):" << seqname << endl;
    m_log.popMessageLevel();
  }
}


string ReadMerger::lookupSeqId(const string & seqname) const
{
  string retval = "";

  ReadMap::const_iterator i = m_readMap.find(seqname);
  if (i != m_readMap.end())
  {
    retval = i->second;
  }

  return retval;
}

string ReadMerger::renumberIds(SliceData * slice)
{
  string newids = "";

  unsigned short depth = slice->getDepth();
  for (unsigned short i = 0; i < depth; i++)
  {
    if (i) { newids += " "; }

    const string & seqname = slice->getRead(i)->getSeqname();
    ReadMap::const_iterator i = m_readMap.find(seqname);
    newids += i->second;
  }

  return newids;
}
