// $Id: Foundation.hh,v 1.1.1.1 2005/08/04 14:50:59 mcschatz Exp $

//! @file 
//! @brief Foundation class interface

#ifndef Foundation_HH
#define Foundation_HH 1

#include <string>
#include "Options.hh"
#include "ConfigFile.hh"
#include "Logger.hh"
#include "FileSystem.hh"

//! Wrapper Class to ease use of Options, Configuration Files, and Logging
class Foundation
{
public:
  Foundation(std::string version, 
             std::string helptext,
             std::string dependencies,
             int argc,
             char ** argv);
  ~Foundation();

  void handleStandardOptions();
  void printHelpText();

  std::string getVersion();
  std::string getHelpText();

  Options * getOptions();
  ConfigFile * getConfig();
  Logger * getLogger();

  LogCategory log();

  void disableOptionHelp();

private:
  std::string m_version;
  std::string m_helptext;
  std::string m_dependencies;

  Options * m_options;
  Logger * m_logger;
  ConfigFile * m_config;

  bool m_doOptionHelp;

  // Command line option results
  int m_doHelp;
  int m_doVersion;
  int m_doAppend;
  int m_doDepend;

  std::string m_configFile;
  std::string m_logfile;
  std::string m_filterLevels;
};

#endif
